#*************************************************************************
# crt0.S
#
# (C) Copyright 2001 to 2001 Elegent Technologies Inc.
#
# July, 23 Mark Lu	File created.
#
# Initialization of ARC processor
#*************************************************************************
.file   "crt0.s"
.global _start
.type   _start, @function
.align	4
.text

.equ	DEBUG_ENABLE,	1		# 1=Enable debugging

#*************************************************************************
# MPEG Adress space definition
#*************************************************************************
.equ	MPEG_BASE,	0xFFE00000	# MPEG base address
.equ	MPEG_HIF_IMR,	0xFFF00080	# HIF register base address
.equ	MPEG_ICC_BASE,	0xFFED0000	# ICC register base address
.equ	MPEG_GPIO_8,	0xFFEE0017	# GPIO register 8 address
.equ	MPEG_GPIO_9,	0xFFEE0016	# GPIO register 9 address
.equ	MPEG_GPIO_10,	0xFFEE0015	# GPIO register 10 address

#*************************************************************************
# ICC register address definition
#*************************************************************************
.equ	ICC_GCNT,	0xFFED0000	# Global control register
.equ	ICC_STAT,	0xFFED0001	# Status register
.equ	ICC_PCNT,	0xFFED0002	# Port control register
.equ	ICC_DATA,	0xFFED0003	# UART data register
.equ	ICC_BAUD,	0xFFED0005	# Baudrate select register
.equ	ICC_GUARD,	0xFFED0006	# Guard time register
.equ	ICC_IDLET,	0xFFED0007	# Idle time register

#*************************************************************************
# A output char macro
#*************************************************************************
.macro	PUT_CHAR	c
.if	DEBUG_ENABLE			#
	or	r1, 0x0C01, 0x0010	# Enable output
	sr	r1, [ICC_PCNT]		# Store r1 to port control register
	mov	r2, \c			# Move output character to r2
	nop				# Nop ==> for pipeline
	sr	r2, [ICC_DATA]		# Output data
x\@:					# Jump label
	lr	r3, [ICC_STAT]		# Read current status
	nop				# Nop ==> for pipeline
	and.f	r3, r3, 0x0008		# 1=Transmitter is sending a char
	nop				# Nop ==> for pipeline
	jnz	x\@			# Wait until character send
.endif					#
.endm					#

#*************************************************************************
# A output string macro
#*************************************************************************
.macro	PUT_STRING	string
.if	DEBUG_ENABLE			#
	
.endif					#
.endm					#

#*************************************************************************
# Program Start
#*************************************************************************
_start:
real_start:
	flag	0			# Disable interrupt
	mov	r0, 0x00000000		# r0 = 0
	mov	r1, 0x00000000		# r1 = 0
	mov	r2, 0x00000000		# r2 = 0
	mov	r3, 0x00000000		# r3 = 0
	mov	r4, 0x00000000		# r4 = 0
	mov	r5, 0x00000000		# r5 = 0
	mov	r6, 0x00000000		# r6 = 0
	mov	r7, 0x00000000		# r7 = 0
	mov	r8, 0x00000000		# r8 = 0
	mov	r9, 0x00000000		# r9 = 0
	mov	r10, 0x00000000		# r10 = 0
	mov	r11, 0x00000000		# r11 = 0
	mov	r12, 0x00000000		# r12 = 0
	mov	r13, 0x00000000		# r13 = 0
	mov	r14, 0x00000000		# r14 = 0
	mov	r15, 0x00000000		# r15 = 0
	mov	r16, 0x00000000		# r16 = 0
	mov	r17, 0x00000000		# r17 = 0
	mov	r18, 0x00000000		# r18 = 0
	mov	r19, 0x00000000		# r19 = 0
	mov	r20, 0x00000000		# r10 = 0
	mov	r21, 0x00000000		# r21 = 0
	mov	r22, 0x00000000		# r22 = 0
	mov	r23, 0x00000000		# r23 = 0
	mov	r24, 0x00000000		# r24 = 0
	mov	r25, 0x00000000		# r25 = 0
	mov	r26, 0x00000000		# r26 = 0
	mov	fp , 0x00000000		# fp = r27 = 0
	mov	sp , _stack_high	# init stack high
	sr	r0, [MPEG_GPIO_10]	# Light on the LED4
.if	DEBUG_ENABLE			#
	mov	r1, 0x0C01		# Set port control register
	sr	r1, [ICC_PCNT]		#
	mov	r2, 0x0000		# Set global control register
	sr	r2, [ICC_GCNT]		#
	mov	r3, 0x002E		# Set 38400 baud rate
	sr	r3, [ICC_BAUD]		#
	mov	r4, 0x0040		# Set guard time
	sr	r4, [ICC_GUARD]		#
	mov	r5, 0x0040		# Set idle time
	sr	r5, [ICC_IDLET]		#
	lr	r6, [ICC_DATA]		# To un-block output
	and	r6, r6, 0x00FF		#
	PUT_CHAR 0x41			# Output a char
.endif					#

	bl.nd	_etmain			# call etmain function
	bl.nd	_test1

  	flag	1			# Halt ARC
	nop				#
	nop				#
	nop				#

#*************************************************************************
# Test1 Subroutine
#*************************************************************************
_test1:
	st	blink, [sp, 4]		# save return address
	st	fp, [sp]		# save fp
	mov	fp, sp			# fp = sp
	sub	sp, sp, 16		# PROLOG end

	bl.nd	_test2			# call test2 function
	
	ld	blink, [fp, 4]		# restore return address
	j.d	blink			# return back and execute next
	ld.a	fp, [sp, 16]		# next instruction
#*************************************************************************
# Test2 Subroutine
#*************************************************************************
_test2:
	st	blink, [sp, 4]		# save return address
	st	fp, [sp]		# save fp
	mov	fp, sp			# fp = sp
	sub	sp, sp, 16		# PROLOG end

	nop			# 

	ld	blink, [fp, 4]		# restore return address
	j.d	blink			# return back and execute next
	ld.a	fp, [sp, 16]		# next instruction
